# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

* [`openssl`](#openssl): Manage X.509 certificates, keys and Diffie-Hellman parameter files

### Defined types

* [`openssl::cacert`](#opensslcacert): Manage an X.509 CA certificate file in PEM format
* [`openssl::cert`](#opensslcert): Manage an X.509 certificate file in PEM format
* [`openssl::csr`](#opensslcsr): Create OpenSSL certificate signing request (CSR)
* [`openssl::dhparam`](#openssldhparam): Manage Diffie-Hellman parameter files
* [`openssl::key`](#opensslkey): Manage an X.509 key file in PEM format

### Resource types

* [`openssl_certutil`](#openssl_certutil): Manage trusted certificates in the system-wide NSS database
* [`openssl_genparam`](#openssl_genparam): Generate Diffie-Hellman or Elliptic Curve parameter file
* [`openssl_genpkey`](#openssl_genpkey): Generate OpenSSL private key files
* [`openssl_hash`](#openssl_hash): Manage a symbolic link using the certificate hash
* [`openssl_selfsign`](#openssl_selfsign): Create an OpenSSL self-signed certificate
* [`openssl_signcsr`](#openssl_signcsr): Sign OpenSSL certificate signing request using a CA

### Data types

* [`Openssl::Keyusage`](#opensslkeyusage): Valid parameter values for the OpenSSL keyusage

## Classes

### <a name="openssl"></a>`openssl`

Manage X.509 certificates, keys and Diffie-Hellman parameter files

#### Examples

##### Declaring the class

```puppet

class { 'openssl':
  cert_source_directory => '/etc/puppetlabs/code/private/certs',
}
```

##### Declaring the class and deploy a CA certificate

```puppet

class { 'openssl':
  cert_source_directory => '/etc/puppetlabs/code/private/certs',
  root_ca_certs         => [ 'ACME-Root-CA' ],
}
```

#### Parameters

The following parameters are available in the `openssl` class:

* [`cert_source_directory`](#cert_source_directory)
* [`default_key_dir`](#default_key_dir)
* [`default_cert_dir`](#default_cert_dir)
* [`package_name`](#package_name)
* [`package_ensure`](#package_ensure)
* [`root_group`](#root_group)
* [`ca_certs`](#ca_certs)

##### <a name="cert_source_directory"></a>`cert_source_directory`

Data type: `Stdlib::Absolutepath`

The directory on the Puppetmaster where all certificate and key files are
kept. Every certificate or key file will be read from this directory and
then deployed on the client. This directory is accessed using the `file`
function and therefore does not need to be part of the Puppet directory
structure. But obviously the directory and the files must be readable by
the Puppet user. This parameter is mandatory and has no default.

##### <a name="default_key_dir"></a>`default_key_dir`

Data type: `Stdlib::Absolutepath`

The default directory where a key file is deployed. This is operating
system specific. On Debian this is `/etc/ssl/private` and on RedHat this
is `/etc/pki/tls/private`.

##### <a name="default_cert_dir"></a>`default_cert_dir`

Data type: `Stdlib::Absolutepath`

The default directory where a certificate file is deployed. This is
operating system specific. On Debian this is `/etc/ssl/certs` and on
RedHat this is `/etc/pki/tls/certs`.

##### <a name="package_name"></a>`package_name`

Data type: `String`

The name of the OpenSSL package to install.

##### <a name="package_ensure"></a>`package_ensure`

Data type: `String`

The desired package state.

##### <a name="root_group"></a>`root_group`

Data type: `String`

The group used for deployed files. This is operating system specific. On
Linux this is normally `root`. On FreeBSD this is `wheel`.

##### <a name="ca_certs"></a>`ca_certs`

Data type: `Array[String]`

An array of CA certificates that are installed by default. Internally
this uses the `openssl::cert` defined type.

## Defined types

### <a name="opensslcacert"></a>`openssl::cacert`

Manage an X.509 CA certificate file in PEM format

#### Examples

##### Install the 'my-root-ca' trusted cert in the default location

```puppet

openssl::cacert { 'my-root-ca': }
```

#### Parameters

The following parameters are available in the `openssl::cacert` defined type:

* [`ensure`](#ensure)
* [`cert`](#cert)
* [`source`](#source)
* [`extension`](#extension)
* [`source_extension`](#source_extension)
* [`mode`](#mode)
* [`owner`](#owner)
* [`group`](#group)
* [`cert_dir`](#cert_dir)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="cert"></a>`cert`

Data type: `String`

The basename of the file where the certificate will be stored on the
client. The full filename will be created using the three components
`cert_dir`, `cert` and `extension`.

Default value: `$name`

##### <a name="source"></a>`source`

Data type: `String`

The basename of the file where the certificate is stored on the server.
The full filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client. This parameter
is ignored on Debian and RedHat based distributions as the operating
system specific tools require certificates to be installed using the
`.crt` extension.

Default value: `'crt'`

##### <a name="source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'crt'`

##### <a name="mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource. Note that certificate
verification may fail if the file permissions are too restrictive.

Default value: `'0444'`

##### <a name="owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: ``undef``

##### <a name="cert_dir"></a>`cert_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the certificate will be
stored. This parameter is ignored on Debian and RedHat based
distributions. Debian requires CA certificates to be stored in
`/usr/local/share/ca-certificates` and RedHat requires CA certificates
to be stored in `/etc/pki/ca-trust/source/anchors`.

Default value: ``undef``

### <a name="opensslcert"></a>`openssl::cert`

Manage an X.509 certificate file in PEM format

#### Examples

##### Install the 'imap' cert in the default location

```puppet

openssl::cert { 'imap': }
```

##### Install the 'postgresql' cert using application specific defaults

```puppet

openssl::cert { 'postgresql':
  cert     => $::hostname,
  owner    => 'root',
  group    => 'postgres',
  mode     => '0444',
  cert_dir => '/etc/postgresql',
  source   => $::hostname,
}
```

#### Parameters

The following parameters are available in the `openssl::cert` defined type:

* [`ensure`](#ensure)
* [`cert`](#cert)
* [`source`](#source)
* [`cert_chain`](#cert_chain)
* [`extension`](#extension)
* [`source_extension`](#source_extension)
* [`mode`](#mode)
* [`owner`](#owner)
* [`group`](#group)
* [`cert_dir`](#cert_dir)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="cert"></a>`cert`

Data type: `String`

The basename of the file where the certificate will be stored on the
client. The full filename will be created using the three components
`cert_dir`, `cert` and `extension`.

Default value: `$name`

##### <a name="source"></a>`source`

Data type: `String`

The basename of the file where the certificate is stored on the server.
The full filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="cert_chain"></a>`cert_chain`

Data type: `Array[String]`

An array of certificate names that are should be added to the certificate
file. This allows the generation of certificate chains to provide a full
verification path for the certificate if intermediate CAs are used. The
chain is included in the generated certificate file. The certificates
must be available in `cert_source_directory` on the server just like the
ordinary certificate.

Default value: `[]`

##### <a name="extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client.

Default value: `'crt'`

##### <a name="source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'crt'`

##### <a name="mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0444'`

##### <a name="owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: ``undef``

##### <a name="cert_dir"></a>`cert_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the certificate will be
stored.

Default value: ``undef``

### <a name="opensslcsr"></a>`openssl::csr`

Create OpenSSL certificate signing request (CSR)

#### Examples

##### Creating a CSR with subject alternate names

```puppet

openssl::csr { '/etc/ssl/www.example.com.csr':
  common_name                 => 'www.example.com',
  subject_alternate_names_dns => [ 'www.example.com', 'example.com', ],
  config                      => '/etc/ssl/www.example.com.cnf',
  key_file                    => '/etc/ssl/www.example.com.key',
}
```

#### Parameters

The following parameters are available in the `openssl::csr` defined type:

* [`common_name`](#common_name)
* [`csr_file`](#csr_file)
* [`config`](#config)
* [`key_file`](#key_file)
* [`subject_alternate_names_dns`](#subject_alternate_names_dns)
* [`subject_alternate_names_ip`](#subject_alternate_names_ip)
* [`key_usage`](#key_usage)
* [`extended_key_usage`](#extended_key_usage)
* [`basic_constraints_ca`](#basic_constraints_ca)
* [`mode`](#mode)
* [`owner`](#owner)
* [`group`](#group)
* [`country_name`](#country_name)
* [`state_or_province_name`](#state_or_province_name)
* [`locality_name`](#locality_name)
* [`postal_code`](#postal_code)
* [`street_address`](#street_address)
* [`organization_name`](#organization_name)
* [`organization_unit_name`](#organization_unit_name)

##### <a name="common_name"></a>`common_name`

Data type: `String`

The value of the X.509 `CN` attribute. This attribute is mandatory.

##### <a name="csr_file"></a>`csr_file`

Data type: `Stdlib::Absolutepath`

The full path name of the signing request file that will be created. It
contains the attributes that will be included in the certificate and also
the public part of the key. Default is the name of the resource.

Default value: `$name`

##### <a name="config"></a>`config`

Data type: `Stdlib::Absolutepath`

The full path name of the OpenSSL configuration file that will be
created. It contains a minimal set of configuration options that are
needed to process the CSR. It can also be used when the CSR is used to
create a self-signed certificate. Updates to the config file will not
trigger the generation of a new certificate.

##### <a name="key_file"></a>`key_file`

Data type: `Stdlib::Absolutepath`

The full path of the private key file. This file including the key must
already be present to generate the CSR.

##### <a name="subject_alternate_names_dns"></a>`subject_alternate_names_dns`

Data type: `Array[Stdlib::Fqdn]`

An array of DNS names that will be added as subject alternate names using
the `DNS` prefix. The certificate can be used for all names given in this
list. Normally the common name should be in this list or the certificate
may be rejected by modern web browsers.

Default value: `[]`

##### <a name="subject_alternate_names_ip"></a>`subject_alternate_names_ip`

Data type: `Array[Stdlib::IP::Address]`

An array of IP addresses that will be added as subject alternate names
using the `IP` prefix. The certificate can be used for all IP addresses
given in this list.

Default value: `[]`

##### <a name="key_usage"></a>`key_usage`

Data type: `Array[Openssl::Keyusage]`

The intended purposes of the certificate.

Default value: `[ 'keyEncipherment', 'dataEncipherment' ]`

##### <a name="extended_key_usage"></a>`extended_key_usage`

Data type: `Array[String]`

The extended key usage of the certificate.

Default value: `[ 'serverAuth' ]`

##### <a name="basic_constraints_ca"></a>`basic_constraints_ca`

Data type: `Boolean`

Whether the subject of the certificate is a CA.

Default value: ``false``

##### <a name="mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0444'`

##### <a name="owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: ``undef``

##### <a name="country_name"></a>`country_name`

Data type: `Optional[String]`

The value of the X.509 `C` attribute.

Default value: ``undef``

##### <a name="state_or_province_name"></a>`state_or_province_name`

Data type: `Optional[String]`

The value of the X.509 `ST` attribute.

Default value: ``undef``

##### <a name="locality_name"></a>`locality_name`

Data type: `Optional[String]`

The value of the X.509 `L` attribute.

Default value: ``undef``

##### <a name="postal_code"></a>`postal_code`

Data type: `Optional[String]`

The value of the X.509 `PC` attribute.

Default value: ``undef``

##### <a name="street_address"></a>`street_address`

Data type: `Optional[String]`

The value of the X.509 `STREET` attribute.

Default value: ``undef``

##### <a name="organization_name"></a>`organization_name`

Data type: `Optional[String]`

The value of the X.509 `O` attribute.

Default value: ``undef``

##### <a name="organization_unit_name"></a>`organization_unit_name`

Data type: `Optional[String]`

The value of the X.509 `OU` attribute.

Default value: ``undef``

### <a name="openssldhparam"></a>`openssl::dhparam`

Manage Diffie-Hellman parameter files

#### Examples

##### Create a parameter file using default parameters

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem': }
```

##### Create a parameter file using 4096 bits

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem':
  bits => '4096',
}
```

##### Create a parameter file using non-default permissions

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem':
  owner => 'www-data',
  group => 'www-data',
  mode  => '0640',
}
```

#### Parameters

The following parameters are available in the `openssl::dhparam` defined type:

* [`ensure`](#ensure)
* [`file`](#file)
* [`bits`](#bits)
* [`generator`](#generator)
* [`mode`](#mode)
* [`owner`](#owner)
* [`group`](#group)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="file"></a>`file`

Data type: `Stdlib::Absolutepath`

The file name where the DH parameters are stored on the node. Must be an
absolute path.

Default value: `$name`

##### <a name="bits"></a>`bits`

Data type: `Enum['2048','4096','8192']`

The number of bits to generate.

Default value: `'2048'`

##### <a name="generator"></a>`generator`

Data type: `Enum['2','5']`

The generator to use. Check the OpenSSL documentation for details about
this parameter.

Default value: `'2'`

##### <a name="mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0644'`

##### <a name="owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: ``undef``

### <a name="opensslkey"></a>`openssl::key`

Manage an X.509 key file in PEM format

#### Examples

##### Install the 'imap' key in the default location

```puppet

openssl::key { 'imap': }
```

##### Install the 'postgresql' key using application specific defaults

```puppet

openssl::key { 'postgresql':
  key     => $::hostname,
  owner   => 'root',
  group   => 'postgres',
  mode    => '0440',
  key_dir => '/etc/postgresql',
  source  => $::hostname,
}
```

#### Parameters

The following parameters are available in the `openssl::key` defined type:

* [`ensure`](#ensure)
* [`key`](#key)
* [`source`](#source)
* [`extension`](#extension)
* [`source_extension`](#source_extension)
* [`mode`](#mode)
* [`owner`](#owner)
* [`group`](#group)
* [`key_dir`](#key_dir)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="key"></a>`key`

Data type: `String`

The basename of the file where the key will be stored on the client. The
full filename will be created using the three components `key_dir`, `key`
and `extension`.

Default value: `$name`

##### <a name="source"></a>`source`

Data type: `String`

The basename of the file where the key is stored on the server. The full
filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client.

Default value: `'key'`

##### <a name="source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'key'`

##### <a name="mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0400'`

##### <a name="owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: ``undef``

##### <a name="key_dir"></a>`key_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the key will be stored.

Default value: ``undef``

## Resource types

### <a name="openssl_certutil"></a>`openssl_certutil`

This type installs the certificate specified with `filename` as a trusted
certificate if `ensure => present`. The trust is removed if `ensure =>
absent`.

The `certutil` executable is required for this type. In general it is
only available on RedHat-based distributions.

The certificate file itself is not managed by this type.

The file must already exist on the node before it can be added to the NSS
database. Make sure you add the correct dependency if you manage the
certificate file with Puppet.

There is an unsolved issue if a certificate is added a second time to the
NSS database using a different name. In this case `certutil` does not add
the certificate but also does not report an error. Therefore Puppet will
try to add the certificate every time it runs. As a workaround the
already installed certificate should be removed.

#### Examples

##### Add a certificate to the NSS database and set trust level for SSL

```puppet

openssl_certutil { '/etc/ssl/certs/My-Root-CA.crt':
  ensure    => present,
  ssl_trust => 'C',
}
```

##### Remove a certificate from the NSS database

```puppet

openssl_certutil { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => absent,
}
```

#### Properties

The following properties are available in the `openssl_certutil` type.

##### `email_trust`

Valid values: `%r{[pPcCT]*}`

Email trust attributes for the certificate.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

##### `object_signing_trust`

Valid values: `%r{[pPcCT]*}`

Object signing trust attributes for the certificate.

##### `ssl_trust`

Valid values: `%r{[pPcCT]*}`

SSL trust attributes for the certificate.

#### Parameters

The following parameters are available in the `openssl_certutil` type.

* [`filename`](#filename)
* [`name`](#name)
* [`provider`](#provider)

##### <a name="filename"></a>`filename`

The filename of the certificate.

##### <a name="name"></a>`name`

namevar

The nickname of the certificate in the certificate database.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_certutil` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_genparam"></a>`openssl_genparam`

The type is refreshable. The `openssl_genaram` type will regenerate the
parameters if the resource is notified from another resource.

#### Examples

##### Create a Diffie-Hellman parameter file using 2048 bits

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm => 'DH',
  bits      => '2048,
  generator => '2',
}
```

##### Create an Elliptic Curve parameter file using the secp521e1 curve

```puppet

openssl_genparam { '/tmp/ecparam.pem':
  algorithm => 'EC',
  curve     => 'secp521r1',
}
```

##### Automatically refresh a parameter file every 3 months

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm        => 'DH',
  bits             => '2048,
  generator        => '2',
  refresh_interval => '3mo',
}
```

##### Refresh a parameter file if another file changes

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm => 'DH',
  bits      => '2048,
  subscribe => File['/etc/ssl/parameters.trigger'],
}
```

#### Properties

The following properties are available in the `openssl_genparam` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_genparam` type.

* [`algorithm`](#algorithm)
* [`bits`](#bits)
* [`curve`](#curve)
* [`file`](#file)
* [`generator`](#generator)
* [`provider`](#provider)
* [`refresh_interval`](#refresh_interval)

##### <a name="algorithm"></a>`algorithm`

Valid values: `DH`, `EC`

The algorithm to generate the parameters for.

##### <a name="bits"></a>`bits`

Valid values: `2048`, `4096`, `8192`

The number of bits to use for Diffie-Hellman parameters.

##### <a name="curve"></a>`curve`

Valid values: `%r{^[a-zA-Z][a-zA-Z0-9-]+[0-9]$}`

The name of the curve to use for Elliptic Curve parameters.

##### <a name="file"></a>`file`

The name of the parameter file to manage.

##### <a name="generator"></a>`generator`

Valid values: `2`, `5`

The generator to use for Diffie-Hellman parameters.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_genparam` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="refresh_interval"></a>`refresh_interval`

Valid values: `%r{^[0-9]+(y|mo|w|d|h|mi|s)?$}`

The Refresh interval for the parameter file. A new parameter file
will be generated after this time.

The value must be a number optionally followed by a time unit. The
following units are understood: `y` for year (365 days), `mo` for
months (30 days), `w` for week (7 days), `d` for days (24 hours), `h`
for hours (60 minutes), `mi` for minute (60 seconds). When the unit `s`
or no unit is used then the value is interpreted as the number of
seconds.

### <a name="openssl_genpkey"></a>`openssl_genpkey`

**This type is still beta!**

Generate an OpenSSL private key file. The type creates RSA or Elliptic
Curve keys depending on the parameter `algorithm`.

The key can optionally be encrypted using a supplied password. Encryption
uses the `-passin` option when calling `openssl` so the password is not
visible in the process listing.

The type is refreshable. The `openssl_genpkey` type will regenerate the
key if the resource is notified from another resource.

#### Examples

##### Generate a 2048 bit RSA key file

```puppet

openssl_genpkey { '/tmp/rsa-2048.key':
  algorithm => 'RSA',
  bits      => '2048',
}
```

##### Generate AES encrypted Elliptic Curve private key

```puppet

openssl_genpkey { '/tmp/ec-secp256k1.key':
  algorithm => 'EC',
  curve     => 'secp256k1',
  cipher    => 'aes128',
  password  => 'rosebud',
}
```

##### Regenerate the key if another file changes

```puppet

openssl_genpkey { '/tmp/rsa-2048.key':
  algorithm => 'RSA',
  bits      => '2048',
  subscribe => File['/etc/ssl/key.trigger'],
}
```

#### Properties

The following properties are available in the `openssl_genpkey` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_genpkey` type.

* [`algorithm`](#algorithm)
* [`bits`](#bits)
* [`cipher`](#cipher)
* [`curve`](#curve)
* [`file`](#file)
* [`password`](#password)
* [`provider`](#provider)

##### <a name="algorithm"></a>`algorithm`

Valid values: `RSA`, `EC`

The algorithm to generate a private key for. The number of bits
must be supplied if an RSA key is generated. For an EC key the curve
name must be given

##### <a name="bits"></a>`bits`

Valid values: `2048`, `4096`, `8192`

The number of bits for the RSA key. This parameter is mandatory for
RSA keys.

##### <a name="cipher"></a>`cipher`

Encrypt the key with the supplied cipher. A password must be given
in this case.

##### <a name="curve"></a>`curve`

Valid values: `%r{^[a-zA-Z][a-zA-Z0-9-]+[0-9]$}`

The curve to use for elliptic curve key. This parameter is
mandatory for EC keys.

##### <a name="file"></a>`file`

The name of the private key file to manage.

##### <a name="password"></a>`password`

Use the supplied password when encrypting the key.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_genpkey` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_hash"></a>`openssl_hash`

If `ensure => present` a symbolic link using the certificate hash will be
created in the same directory as the certificate. The link is removed if
`ensure => absent`.

This link is used to find a trusted cert when a certificate chain is
validated.

The certificate file itself is not managed by this type.

The file must exist before the link can be created as it is accessed by
OpenSSL to calculate the hash. For the same reason the file can only be
deleted after the link has been removed.

#### Examples

##### Mark an existing certificate as trusted

```puppet

openssl_trustcert { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => present,
}
```

##### Mark an existing certificate as not trusted

```puppet

openssl_trustcert { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => absent,
}
```

#### Properties

The following properties are available in the `openssl_hash` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_hash` type.

* [`name`](#name)
* [`provider`](#provider)

##### <a name="name"></a>`name`

namevar

The name of the certificate file to manage.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_hash` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_selfsign"></a>`openssl_selfsign`

**This type is still beta!**

The type takes a certificate signing request (CSR) and a key file to
generate a self-signed certificate.

Certificate extensions can be added by using the `extensions` and
`extfile` parameters.

Optionally a key password can be provided if the used key is encrypted.

The certificate will be valid for the given number of days.

The type is refreshable. The `openssl_selfsign` type will regenerate the
certificate if the resource is notified from another resource.

#### Examples

##### Create a self-signed certificate with extensions valid for one year

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr        => '/tmp/cert.csr',
  signkey    => '/tmp/cert.key',
  extfile    => '/tmp/cert.cnf',
  extensions => 'v3_ext',
  days       => '365',
}
```

#### Properties

The following properties are available in the `openssl_selfsign` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_selfsign` type.

* [`csr`](#csr)
* [`days`](#days)
* [`extensions`](#extensions)
* [`extfile`](#extfile)
* [`file`](#file)
* [`password`](#password)
* [`provider`](#provider)
* [`signkey`](#signkey)

##### <a name="csr"></a>`csr`

Required. The file containing the certificate signing request.

##### <a name="days"></a>`days`

Valid values: `%r{^[0-9]+$}`

The number of days the certificate should be valid.

Default value: `370`

##### <a name="extensions"></a>`extensions`

The section name of the extensions. The OpenSSL defaults will be used
if the parameter is `undef`.

##### <a name="extfile"></a>`extfile`

The file with the certificate extensions.

##### <a name="file"></a>`file`

The signed certificate file to manage.

##### <a name="password"></a>`password`

The password to decrypt the key.
Leave the parameter undefined if the key is not encrypted.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_selfsign` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="signkey"></a>`signkey`

Required. The file with the OpenSSL key to use for the self-signed certificate.

### <a name="openssl_signcsr"></a>`openssl_signcsr`

**This type is still beta!**

The name and configuration file of a CA is required.

Certificate extensions can be added by using the `extensions` and
`extfile` parameters.

Optionally a key password can be provided if the used key is encrypted.

The certificate will be valid for the given number of days.

The type is refreshable. The `openssl_signcsr` type will regenerate the
certificate if the resource is notified from another resource.

#### Examples

##### Use a CA to sign a CSR

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr       => '/tmp/cert.csr',
  ca_name   => 'My-Root-CA',
  ca_config => '/etc/ssl/CA.cnf',
  days      => '365',
}
```

##### Regenerate a certificate if the CSR changes

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr       => '/tmp/cert.csr',
  ca_name   => 'My-Root-CA',
  ca_config => '/etc/ssl/CA.cnf',
  subscribe => File['/tmp/cert.csr'],
}
```

#### Properties

The following properties are available in the `openssl_signcsr` type.

##### `ensure`

Valid values: `present`, `absent`

Specifies whether the resource should exist.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_signcsr` type.

* [`ca_config`](#ca_config)
* [`ca_name`](#ca_name)
* [`csr`](#csr)
* [`days`](#days)
* [`extensions`](#extensions)
* [`extfile`](#extfile)
* [`file`](#file)
* [`password`](#password)
* [`provider`](#provider)

##### <a name="ca_config"></a>`ca_config`

Required. The configuration file of the CA that is used to sign the CSR.

##### <a name="ca_name"></a>`ca_name`

Required. The name of the CA that is used to sign the CSR.

##### <a name="csr"></a>`csr`

Required. The file containing the certificate signing request.

##### <a name="days"></a>`days`

Valid values: `%r{^[0-9]+$}`

The number of days the certificate should be valid.

Default value: `370`

##### <a name="extensions"></a>`extensions`

The section name of the extensions. The OpenSSL defaults will be used
if the parameter is `undef`.

##### <a name="extfile"></a>`extfile`

The file with the certificate extensions.

##### <a name="file"></a>`file`

The signed certificate file to manage.

##### <a name="password"></a>`password`

The password to decrypt the CA key.
Leave the parameter undefined if the key is not encrypted.

##### <a name="provider"></a>`provider`

The specific backend to use for this `openssl_signcsr` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

## Data types

### <a name="opensslkeyusage"></a>`Openssl::Keyusage`

Valid parameter values for the OpenSSL keyusage

Alias of

```puppet
Enum['digitalSignature', 'nonRepudiation', 'keyEncipherment', 'dataEncipherment', 'keyAgreement', 'keyCertSign', 'cRLSign', 'encipherOnly', 'decipherOnly']
```

